package cn.com.qmth.examcloud.core.reports.base.util.excel;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/*
 * excel导出工具
 */
public class ExportUtils {
	private static final Logger LOG = LoggerFactory.getLogger(ExportUtils.class);
	private static final String DEFALUT_CONTENT_TYPE = "application/vnd.ms-excel";

	private static final String DEFALUT_EXT = ".xlsx";

	public static void exportExcel(String fileName, List<SheetData> data, HttpServletResponse response) {
		try {

			response.setHeader("Content-Disposition",
					"inline; filename=" + URLEncoder.encode(fileName, "UTF-8") + DEFALUT_EXT);
			response.setContentType(DEFALUT_CONTENT_TYPE);
			ServletOutputStream outputStream = response.getOutputStream();
			write(data, outputStream);
			outputStream.flush();
			outputStream.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static void write(List<SheetData> data, OutputStream out) {
		// 声明一个工作薄
		XSSFWorkbook workbook = new XSSFWorkbook();
		try {
			for (SheetData sheetData : data) {
				// 生成一个表格
				XSSFSheet sheet = workbook.createSheet(sheetData.getName());

				// 生成一个样式
				XSSFCellStyle style = workbook.createCellStyle();

				// 产生表格标题行
				List<String> header = sheetData.getHeader();
				int[] colwidth = new int[header.size()];
				XSSFRow row = sheet.createRow(0);
				for (short i = 0; i < header.size(); i++) {
					colwidth[i] = 15;
					XSSFCell cell = row.createCell(i);
					cell.setCellStyle(style);
					XSSFRichTextString text = new XSSFRichTextString(header.get(i));
					cell.setCellValue(text);
					int textlen = length(header.get(i));
					if (textlen > colwidth[i]) {
						colwidth[i] = textlen;
					}
				}

				List<Object[]> rows = sheetData.getData();
				// 遍历集合数据，产生数据行
				for (short r = 0; r < rows.size(); r++) {
					row = sheet.createRow(r + 1);
					Object[] rowdata = rows.get(r);
					for (short i = 0; i < header.size(); i++) {
						XSSFCell cell = row.createCell(i);
						Object value = rowdata[i];
						// 判断值的类型后进行强制类型转换
						String textValue = null;
						if (value == null) {
							textValue = "";
						} else if (value instanceof Boolean) {
							boolean bValue = (Boolean) value;
							textValue = "是";
							if (!bValue) {
								textValue = "否";
							}
						} else if (value instanceof Date) {
							Date date = (Date) value;
							SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
							textValue = sdf.format(date);
						} else {
							// 其它数据类型都当作字符串简单处理
							textValue = String.valueOf(value);
						}
						// 利用正则表达式判断textValue是否全部由数字组成
						if (textValue != null) {
							Pattern p = Pattern.compile("^\\d+(\\.\\d+)?$");
							Matcher matcher = p.matcher(textValue);
							if (matcher.matches()) {
								// 是数字当作double处理
								cell.setCellValue(Double.parseDouble(textValue));
							} else {
								XSSFRichTextString richString = new XSSFRichTextString(textValue);

								cell.setCellValue(richString);
							}
						} else {
							textValue = "";
						}
						int textlen = length(textValue);
						if (textlen > colwidth[i]) {
							colwidth[i] = textlen;
						}
					}

				}
				for (int i = 0; i < colwidth.length; i++) {
					sheet.setColumnWidth(i, 286 * colwidth[i]);
				}
			}
			workbook.write(out);
		} catch (Exception e) {
			LOG.error("导出出错", e);
		} finally {
			try {
				workbook.close();
			} catch (IOException e) {
				LOG.error("workbook close 出错", e);
			}
		}

	}

	private static int length(String value) {
		int valueLength = 0;
		String chinese = "[\u4e00-\u9fa5]";
		for (int i = 0; i < value.length(); i++) {
			String temp = value.substring(i, i + 1);
			if (temp.matches(chinese)) {
				valueLength += 2;
			} else {
				valueLength += 1;
			}
		}
		return valueLength;
	}
}
