package cn.com.qmth.examcloud.core.reports.service.impl;

import java.util.Date;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.core.reports.base.bean.StudentCountBean;
import cn.com.qmth.examcloud.core.reports.base.util.online.ActiveDataUtil;
import cn.com.qmth.examcloud.core.reports.base.util.online.OnlineCount;
import cn.com.qmth.examcloud.core.reports.dao.StudentCountRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.StudentCountEntity;
import cn.com.qmth.examcloud.core.reports.service.StudentCountService;

@Service
public class StudentCountServiceImpl implements StudentCountService {
	@Autowired
	private StudentCountRepo studentCountRepo;


	@Transactional
	@Override
	public synchronized void saveCount() {
		studentCountRepo.resetAllCount();
		List<OnlineCount> ret = ActiveDataUtil.getStudentCount(null);
		if (ret != null) {
			for (OnlineCount oc : ret) {
				addOrUpdateCount(oc);
			}
		}
	}

	private void addOrUpdateCount(OnlineCount oc) {
		StudentCountEntity e = studentCountRepo.findByRootOrgId(oc.getRootOrgId());
		if (e == null) {
			e = new StudentCountEntity();
			e.setRootOrgId(oc.getRootOrgId());
			Date now = new Date();
			e.setCreationTime(now);
			e.setUpdateTime(now);
		}
		e.setOnlineCount(oc.getOnlineCount());
		studentCountRepo.save(e);
	}

	@Override
	public List<StudentCountBean> getCountByRootOrgIds(List<Long> rootOrgIds) {
		if (CollectionUtils.isEmpty(rootOrgIds)) {
			return null;
		}
		List<StudentCountEntity> list = studentCountRepo.getCountByRootOrgIds(rootOrgIds);
		if (CollectionUtils.isEmpty(list)) {
			return null;
		}
		List<StudentCountBean> ret = Lists.newArrayList();
		for (StudentCountEntity e : list) {
			StudentCountBean bean = new StudentCountBean();
			bean.setRootOrgId(e.getRootOrgId());
			bean.setOnlineCount(e.getOnlineCount());
			ret.add(bean);
		}
		return ret;
	}

	@Override
	public Long getSumOnlineCount() {
		return studentCountRepo.getSumOnlineCount();
	}
}
